unit ISAPIFilter;
/////////////////////////////////////////////////////////////////////
//                    Delphi ISAPI Filter project
//                Copyright (c) 2002 by Kiril Antonov
/////////////////////////////////////////////////////////////////////
{DEFINE DEBUG}
interface

uses
  Windows, Messages, SysUtils, Classes, SyncObjs, ISAPI2;

type
  TOnGetFilterVersion = Procedure(var pVer: THTTP_FILTER_VERSION; Var Result :Bool) Of Object;
  TOnIISNotification  = Procedure(var pFc: THTTP_FILTER_CONTEXT; NotificationType: DWORD; pvNotification: Pointer; Var Result: DWord) Of Object;
  TOnTerminateFilter  = Procedure(dwFlags: DWORD; var Result : Bool) Of Object;
  TOnReadRawData      = TOnIISNotification;
  TOnSendRawData      = TOnIISNotification;
  TOnPreprocHeaders   = TOnIISNotification;
  TOnAuthentication   = TOnIISNotification;
  TOnURLMap           = TOnIISNotification;
  TOnAccessDenied     = TOnIISNotification;
  TOnLog              = TOnIISNotification;
  TOnEndOfNetSession  = TOnIISNotification;

  TPortNotification   = (NonSecurePort, SecurePort);
  TPortNotificationSet= Set Of TPortNotification;
  TFilterPriority     = (Default, Low, Medium, High);

  TISAPIFilter = class(Tcomponent)
  private
    { Private declarations }
    F_FilterDescription  : String;
    F_OnGetFilterVersion : TOnGetFilterVersion;
    F_OnReadRawData      : TOnReadRawData;
    F_OnSendRawData      : TOnSendRawData;
    F_OnPreprocHeaders   : TOnPreprocHeaders;
    F_OnAuthentication   : TOnAuthentication;
    F_OnURLMap           : TOnURLMap;
    F_OnAccessDenied     : TOnAccessDenied;
    F_OnLog              : TOnLog;
    F_OnEndOfNetSession  : TOnEndOfNetSession;
    F_OnTerminateFilter  : TOnTerminateFilter;
    F_PortNotification   : TPortNotificationSet;
    F_FilterPriority     : TFilterPriority;
  protected
    { Protected declarations }
  public
    { Public declarations }
    Constructor            Create(AOwner:TComponent); override;
    Destructor             Destroy; override;
  published
    { Published declarations }
    Property              FilterDescription   : String               Read F_FilterDescription  Write F_FilterDescription;
    Property              FilterPriority      : TFilterPriority      Read F_FilterPriority     Write F_FilterPriority;
    Property              PortNotification    : TPortNotificationSet Read F_PortNotification   Write F_PortNotification;
    Property              OnGetFilterVersion  : TOnGetFilterVersion  Read F_OnGetFilterVersion Write F_OnGetFilterVersion;
    Property              OnReadRawData       : TOnReadRawData       Read F_OnReadRawData      Write F_OnReadRawData;
    Property              OnSendRawData       : TOnSendRawData       Read F_OnSendRawData      Write F_OnSendRawData;
    Property              OnPreprocHeaders    : TOnPreprocHeaders    Read F_OnPreprocHeaders   Write F_OnPreprocHeaders;
    Property              OnAuthentication    : TOnAuthentication    Read F_OnAuthentication   Write F_OnAuthentication;
    Property              OnURLMap            : TOnURLMap            Read F_OnURLMap           Write F_OnURLMap;
    Property              OnAccessDenied      : TOnAccessDenied      Read F_OnAccessDenied     Write F_OnAccessDenied;
    Property              OnLog               : TOnLog               Read F_OnLog              Write F_OnLog;
    Property              OnEndOfNetSession   : TOnEndOfNetSession   Read F_OnEndOfNetSession  Write F_OnEndOfNetSession;
    Property              OnTerminateFilter   : TOnTerminateFilter   Read F_OnTerminateFilter  Write F_OnTerminateFilter;
  end;

Var
  TheISAPIFilter : TISAPIFilter;
  Section        : TCriticalSection;
  MainDir        : String;
  MainDll        : String;

Function          GetFilterVersion (var pVer: THTTP_FILTER_VERSION): BOOL stdcall; export;
Function          HttpFilterProc   (var pFc: THTTP_FILTER_CONTEXT; Notificationtype: DWORD; pvNotification: Pointer): DWORD stdcall; export;
Function          TerminateFilter  (dwFlags : DWORD): BOOL stdcall; export;

Procedure         LogInfo(LogData:String);
Procedure         Register;

implementation
Var
 Dll : Array[0..1000] of Char;

{ TISAPIFilter }
constructor TISAPIFilter.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  if TheISAPIFilter <> Nil Then
     Raise Exception.Create('Only one instance of ISAPIFilter is permitted per project!');
  TheISAPIFilter       := Self;
  //////////////////////////////////////////////////////////////////////////////
  F_FilterDescription  := '';
  F_PortNotification   := [SecurePort, NonSecurePort];
  F_FilterPriority     := Default;
  //////////////////////////////////////////////////////////////////////////////
  F_OnGetFilterVersion := Nil;
  F_OnReadRawData      := Nil;
  F_OnSendRawData      := Nil;
  F_OnPreprocHeaders   := Nil;
  F_OnAuthentication   := Nil;
  F_OnURLMap           := Nil;
  F_OnAccessDenied     := Nil;
  F_OnLog              := Nil;
  F_OnEndOfNetSession  := Nil;
  F_OnTerminateFilter  := Nil;
end;

destructor TISAPIFilter.Destroy;
begin
  TheISAPIFilter := Nil;
  inherited Destroy;
end;

////////////////////////////////////////////////////////////////////////////////
//                      EXPORTED FUNCIONS REQUIRED BY IIS
////////////////////////////////////////////////////////////////////////////////
Procedure LogInfo(LogData:String);
Var
 S  : String;
 FN : String;
 F  : Text;
 DT : TDateTime;
Begin
  {$I-}
  Section.Acquire;
  Try
   DT:=Now;
   DateTimeToString(S,'ddddd, tt:zzz',DT);
   S:=S+#9+LogData;
   FN:=MainDll+'.log';
   FileMode:=fmShareDenyWrite;
   if FileExists(FN) Then
     Begin
       AssignFile(F,FN);
       Repeat
         Append(F);
       Until (IOResult=0);
     End
   Else
     Begin
        AssignFile(F,FN);
        Repeat
         Rewrite(F);
        Until (IOResult=0);
     End;
   System.Writeln(F,S);
   IOResult;
   CloseFile(F);
   IOResult;
   FileMode:=2;
  Except
  End;
  Section.Release;
  {$I+}
End;


function GetFilterVersion(var pVer: THTTP_FILTER_VERSION): BOOL;
begin
 Section.Acquire;
 Result := True;
 if Assigned(TheISAPIFilter) Then
    Begin
     Try
       StrPCopy(PVer.lpszFilterDesc,System.Copy(TheISAPIFilter.FilterDescription,1,SF_MAX_FILTER_DESC_LEN));
       PVer.dwFilterVersion                                                    := HTTP_FILTER_REVISION;
       PVer.dwFlags                                                            := SF_NOTIFY_ORDER_DEFAULT;
       if TheISAPIFilter.F_FilterPriority = Low              Then PVer.dwFlags := SF_NOTIFY_ORDER_LOW
       Else
       if TheISAPIFilter.F_FilterPriority = Medium           Then PVer.dwFlags := SF_NOTIFY_ORDER_MEDIUM
       Else
       if TheISAPIFilter.F_FilterPriority = High             Then PVer.dwFlags := SF_NOTIFY_ORDER_HIGH;
       if NonSecurePort In TheISAPIFilter.F_PortNotification Then PVer.dwFlags := PVer.dwFlags Or SF_NOTIFY_NONSECURE_PORT;
       if   SecurePort    In TheISAPIFilter.F_PortNotification Then PVer.dwFlags := PVer.dwFlags Or SF_NOTIFY_SECURE_PORT;
       If Assigned(TheISAPIFilter.F_OnReadRawData)           Then PVer.dwFlags := PVer.dwFlags Or SF_NOTIFY_READ_RAW_DATA;
       If Assigned(TheISAPIFilter.F_OnSendRawData)           Then PVer.dwFlags := PVer.dwFlags Or SF_NOTIFY_SEND_RAW_DATA;
       If Assigned(TheISAPIFilter.F_OnPreprocHeaders)        Then PVer.dwFlags := PVer.dwFlags Or SF_NOTIFY_PREPROC_HEADERS;
       If Assigned(TheISAPIFilter.F_OnAuthentication)        Then PVer.dwFlags := PVer.dwFlags Or SF_NOTIFY_AUTHENTICATION;
       If Assigned(TheISAPIFilter.F_OnURLMap)                Then PVer.dwFlags := PVer.dwFlags Or SF_NOTIFY_URL_MAP;
       If Assigned(TheISAPIFilter.F_OnAccessDenied)          Then PVer.dwFlags := PVer.dwFlags Or SF_NOTIFY_ACCESS_DENIED;
       If Assigned(TheISAPIFilter.F_OnLog)                   Then PVer.dwFlags := PVer.dwFlags Or SF_NOTIFY_LOG;
       If Assigned(TheISAPIFilter.F_OnEndOfNetSession)       Then PVer.dwFlags := PVer.dwFlags Or SF_NOTIFY_END_OF_NET_SESSION;
       if Assigned(TheISAPIFilter.F_OnGetFilterVersion)      Then TheISAPIFilter.F_OnGetFilterVersion(pVer,Result);
     Except
       Result := False;
     End;
    End;
 Section.Release;
 {$IFDEF DEBUG}
  LogInfo('Filter is loaded and GetFilterVersion is called ...');
 {$ENDIF}
end;

Function HttpFilterProc (var pFc: THTTP_FILTER_CONTEXT; NotificationType: DWORD; pvNotification: Pointer): DWORD;
Begin
  Section.Acquire;
  Result := SF_STATUS_REQ_NEXT_NOTIFICATION;
  if Assigned(TheISAPIFilter) Then
     Begin
      Try
       Case NotificationType of
            SF_NOTIFY_READ_RAW_DATA       : If Assigned(TheISAPIFilter.F_OnReadRawData)      Then TheISAPIFilter.F_OnReadRawData     (pFc, NotificationType, pvNotification, Result);
            SF_NOTIFY_SEND_RAW_DATA       : If Assigned(TheISAPIFilter.F_OnSendRawData)      Then TheISAPIFilter.F_OnSendRawData     (pFc, NotificationType, pvNotification, Result);
            SF_NOTIFY_PREPROC_HEADERS     : If Assigned(TheISAPIFilter.F_OnPreprocHeaders)   Then TheISAPIFilter.F_OnPreprocHeaders  (pFc, NotificationType, pvNotification, Result);
            SF_NOTIFY_AUTHENTICATION      : If Assigned(TheISAPIFilter.F_OnAuthentication)   Then TheISAPIFilter.F_OnAuthentication  (pFc, NotificationType, pvNotification, Result);
            SF_NOTIFY_URL_MAP             : If Assigned(TheISAPIFilter.F_OnURLMap)           Then TheISAPIFilter.F_OnURLMap          (pFc, NotificationType, pvNotification, Result);
            SF_NOTIFY_ACCESS_DENIED       : If Assigned(TheISAPIFilter.F_OnAccessDenied)     Then TheISAPIFilter.F_OnAccessDenied    (pFc, NotificationType, pvNotification, Result);
            SF_NOTIFY_LOG                 : If Assigned(TheISAPIFilter.F_OnLog)              Then TheISAPIFilter.F_OnLog             (pFc, NotificationType, pvNotification, Result);
            SF_NOTIFY_END_OF_NET_SESSION  : If Assigned(TheISAPIFilter.F_OnEndOfNetSession)  Then TheISAPIFilter.F_OnEndOfNetSession (pFc, NotificationType, pvNotification, Result);
       End;
      Except
        Result := SF_STATUS_REQ_ERROR;
      End;
     End;
  Section.Release;
  {$IFDEF DEBUG}
  LogInfo('Filter called for notification No:'+IntToStr(NotificationType));
  {$ENDIF}
End;

Function TerminateFilter  (dwFlags : DWORD): BOOL;
Begin
  Section.Acquire;
  Result := True;
  if Assigned(TheISAPIFilter) Then
     Begin
      Try
       If Assigned(TheISAPIFilter.F_OnTerminateFilter) Then TheISAPIFilter.F_OnTerminateFilter(dwFlags, Result);
      Except
      End;
     End;
  Section.Release;
  {$IFDEF DEBUG}
  LogInfo('Filter is unloaded ...');
  {$ENDIF}
End;
////////////////////////////////////////////////////////////////////////////////

procedure Register;
begin
  RegisterComponents('Internet', [TISAPIFilter]);
end;

exports
  GetFilterVersion,
  HttpFilterProc,
  TerminateFilter;

Initialization
  TheISAPIFilter      := Nil;
  Section             := TCriticalSection.Create;
  GetModuleFileName   (HInstance,Dll,1000);
  MainDll             := StrPas(Dll);
  MainDir             := ExtractFilePath(MainDll);
Finalization
  TheISAPIFilter := Nil;
  Section.Free;
end.
